"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.observe = observe;
exports.useIsFirst = exports.useInViewPort = void 0;
var _react = _interopRequireWildcard(require("react"));
var _store = require("./store.js");
var _utils = require("./utils.js");
var _jsxRuntime = require("react/jsx-runtime");
function _getRequireWildcardCache(e) { if ("function" != typeof WeakMap) return null; var r = new WeakMap(), t = new WeakMap(); return (_getRequireWildcardCache = function (e) { return e ? t : r; })(e); }
function _interopRequireWildcard(e, r) { if (!r && e && e.__esModule) return e; if (null === e || "object" != typeof e && "function" != typeof e) return { default: e }; var t = _getRequireWildcardCache(r); if (t && t.has(e)) return t.get(e); var n = { __proto__: null }, a = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var u in e) if ("default" !== u && {}.hasOwnProperty.call(e, u)) { var i = a ? Object.getOwnPropertyDescriptor(e, u) : null; i && (i.get || i.set) ? Object.defineProperty(n, u, i) : n[u] = e[u]; } return n.default = e, t && t.set(e, n), n; }
const ConfigurationContext = /*#__PURE__*/(0, _react.createContext)({
  enabled: true
});
const CallbacksContext = /*#__PURE__*/(0, _react.createContext)({
  addInViewPortCallback: () => {},
  removeInViewPortCallback: () => {},
  addIsFirstCallback: () => {},
  removeIsFirstCallback: () => {}
});
const IsInViewPortContext = /*#__PURE__*/(0, _react.createContext)({
  isInViewPort: () => undefined
});
const ItemContext = /*#__PURE__*/(0, _react.createContext)({
  key: undefined
});
const useInViewPort = (callback, deps) => {
  const {
    key
  } = (0, _react.useContext)(ItemContext);
  const {
    addInViewPortCallback,
    removeInViewPortCallback
  } = (0, _react.useContext)(CallbacksContext);
  const finalDeps = Array.isArray(deps) ? deps : [];
  (0, _react.useEffect)(() => {
    if (!key) return; // If it is not an item of observable list.
    addInViewPortCallback(key, callback);
    return () => {
      removeInViewPortCallback(key, callback);
    };
    // The callback depends on deps.
    // eslint-disable-next-line react-hooks/exhaustive-deps
  }, [key, addInViewPortCallback, removeInViewPortCallback, ...finalDeps]);
};
exports.useInViewPort = useInViewPort;
const useIsFirst = (callback, deps) => {
  const {
    key
  } = (0, _react.useContext)(ItemContext);
  const {
    addIsFirstCallback,
    removeIsFirstCallback
  } = (0, _react.useContext)(CallbacksContext);
  const finalDeps = Array.isArray(deps) ? deps : [];
  (0, _react.useEffect)(() => {
    if (!key) return; // If it is not an item of observable list.
    addIsFirstCallback(key, callback);
    return () => {
      removeIsFirstCallback(key, callback);
    };
    // The callback depends on deps.
    // eslint-disable-next-line react-hooks/exhaustive-deps
  }, [key, addIsFirstCallback, removeIsFirstCallback, ...finalDeps]);
};
exports.useIsFirst = useIsFirst;
function observe(List) {
  return /*#__PURE__*/_react.default.forwardRef(function ({
    onViewableItemsChanged,
    keyExtractor,
    renderItem,
    $$enabled,
    ...props
  }, ref) {
    const {
      key
    } = (0, _react.useContext)(ItemContext);
    const {
      isInViewPort
    } = (0, _react.useContext)(IsInViewPortContext);
    const firstKey = (0, _react.useRef)();
    const viewableKeys = (0, _react.useRef)(new Set()).current;

    // traverse to root recursively.
    const isInViewPortRecursively = (0, _react.useCallback)(itemKey => {
      const isInParentViewPort = isInViewPort(key);
      const inViewPort = viewableKeys.has(itemKey);
      if (isInParentViewPort === undefined) {
        return inViewPort;
      }
      return inViewPort && isInParentViewPort;
    }, [isInViewPort, key, viewableKeys]);
    const inViewPortStore = (0, _store.useStore)();
    const isFirstStore = (0, _store.useStore)();
    const addCallbackOrClean = (0, _react.useCallback)((store, itemKey, callback) => {
      if (isInViewPortRecursively(itemKey)) {
        const clean = callback();
        (0, _store.addClean)(store, itemKey, callback, clean);
      } else {
        (0, _store.addCallback)(store, itemKey, callback);
      }
    }, [isInViewPortRecursively]);
    const removeCallbackTasks = (0, _react.useRef)();
    const reserveRemoveCallbackAndClean = (0, _react.useCallback)((store, itemKey, callback) => {
      const task = () => {
        (0, _store.removeCallback)(store, itemKey, callback);
        (0, _store.removeClean)(store, itemKey, callback);
      };
      if (!removeCallbackTasks.current) {
        removeCallbackTasks.current = [];
      }
      removeCallbackTasks.current.push(task);
    }, []);
    const {
      enabled: parentEnabled
    } = (0, _react.useContext)(ConfigurationContext);
    const enabled = parentEnabled && $$enabled !== false ? true : false;
    const enabledRef = (0, _react.useRef)(enabled); //  This is because `FlashList` does not detect changes in `onViewableItemsChanged`.
    enabledRef.current = enabled;

    // When self is an item of an observable list.
    useInViewPort(() => {
      if (!enabled) return;
      viewableKeys.forEach(itemKey => {
        if (isInViewPortRecursively(itemKey)) {
          (0, _store.consumeCallbacks)(itemKey, inViewPortStore);
        }
      });
      return () => {
        viewableKeys.forEach(itemKey => {
          (0, _store.consumeCleans)(itemKey, inViewPortStore);
        });
      };
    }, [enabled]);
    (0, _react.useEffect)(() => {
      // true -> false
      if (!enabled) {
        viewableKeys.forEach(itemKey => {
          (0, _store.consumeCleans)(itemKey, inViewPortStore);
        });
      }
      return () => {
        // false -> true
        if (!enabled) {
          // consume callback
          viewableKeys.forEach(itemKey => {
            if (isInViewPortRecursively(itemKey)) {
              (0, _store.consumeCallbacks)(itemKey, inViewPortStore);
            }
          });
        }
      };
      // eslint-disable-next-line react-hooks/exhaustive-deps
    }, [enabled]);
    const handleViewableKeys = (0, _react.useCallback)(({
      viewableItems
    }) => {
      const willHideKeys = new Set(viewableKeys);
      for (let i = 0; i < viewableItems.length; i++) {
        const viewableItem = viewableItems[i];
        if (!viewableItem) {
          continue;
        }
        const {
          item,
          isViewable
        } = viewableItem;
        if (isViewable) {
          const itemKey = (0, _utils.isFunction)(keyExtractor) ? keyExtractor(item) : item;
          const isNew = !viewableKeys.has(itemKey);
          if (isNew) {
            viewableKeys.add(itemKey);
            if (enabledRef.current) {
              if (isInViewPortRecursively(itemKey)) {
                (0, _store.consumeCallbacks)(itemKey, inViewPortStore);
              }
            }
          }
          willHideKeys.delete(itemKey);
        }
      }
      willHideKeys.forEach(itemKey => {
        viewableKeys.delete(itemKey);
        if (enabledRef.current) {
          (0, _store.consumeCleans)(itemKey, inViewPortStore);
        }
      });
    }, [inViewPortStore, isInViewPortRecursively, keyExtractor, viewableKeys]);
    const handleFirstKey = (0, _react.useCallback)(({
      viewableItems
    }) => {
      const first = viewableItems[0]?.item;
      const firstItemKey = (0, _utils.isFunction)(keyExtractor) ? keyExtractor(first) : first;
      if (enabledRef.current && firstItemKey !== firstKey.current) {
        const prevKey = firstKey.current;
        if (prevKey !== undefined) {
          (0, _store.consumeCleans)(prevKey, isFirstStore);
        }
        firstKey.current = firstItemKey;
        if (isInViewPortRecursively(firstItemKey)) {
          (0, _store.consumeCallbacks)(firstItemKey, isFirstStore);
        }
      }
    }, [isFirstStore, isInViewPortRecursively, keyExtractor]);
    const handleRemoveCallbackTasks = (0, _react.useCallback)(() => {
      if (removeCallbackTasks.current) {
        for (let i = 0; i < removeCallbackTasks.current.length; i++) {
          removeCallbackTasks.current[i]?.();
        }
      }
      removeCallbackTasks.current = undefined;
    }, []);

    // eslint-disable-next-line react-hooks/exhaustive-deps
    const handleViewableItemsChanged = (0, _react.useCallback)((0, _utils.parallelize)(handleRemoveCallbackTasks, handleViewableKeys, handleFirstKey, onViewableItemsChanged), [handleRemoveCallbackTasks, handleViewableKeys, handleFirstKey, onViewableItemsChanged]);
    const ListComponent = List;
    return /*#__PURE__*/(0, _jsxRuntime.jsx)(ConfigurationContext.Provider, {
      value: {
        enabled
      },
      children: /*#__PURE__*/(0, _jsxRuntime.jsx)(IsInViewPortContext.Provider, {
        value: {
          isInViewPort: isInViewPortRecursively
        },
        children: /*#__PURE__*/(0, _jsxRuntime.jsx)(CallbacksContext.Provider, {
          value: {
            addInViewPortCallback: (0, _react.useCallback)((itemKey, callback) => addCallbackOrClean(inViewPortStore, itemKey, callback), [addCallbackOrClean, inViewPortStore]),
            removeInViewPortCallback: (0, _react.useCallback)((itemKey, callback) => reserveRemoveCallbackAndClean(inViewPortStore, itemKey, callback), [reserveRemoveCallbackAndClean, inViewPortStore]),
            addIsFirstCallback: (0, _react.useCallback)((itemKey, callback) => addCallbackOrClean(isFirstStore, itemKey, callback), [addCallbackOrClean, isFirstStore]),
            removeIsFirstCallback: (0, _react.useCallback)((itemKey, callback) => reserveRemoveCallbackAndClean(isFirstStore, itemKey, callback), [reserveRemoveCallbackAndClean, isFirstStore])
          },
          children: /*#__PURE__*/(0, _jsxRuntime.jsx)(ListComponent, {
            ...props,
            ref: ref,
            keyExtractor: keyExtractor,
            onViewableItemsChanged: handleViewableItemsChanged,
            renderItem: (0, _react.useCallback)(itemProps => {
              const itemKey = (0, _utils.isFunction)(keyExtractor) ? keyExtractor(itemProps.item) : itemProps.item;
              return /*#__PURE__*/(0, _jsxRuntime.jsx)(ItemContext.Provider, {
                value: {
                  key: itemKey
                },
                children: renderItem(itemProps)
              });
            }, [keyExtractor, renderItem])
          })
        })
      })
    });
  });
}
//# sourceMappingURL=observe.js.map