"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.useStore = exports.removeClean = exports.removeCallback = exports.consumeCleans = exports.consumeCallbacks = exports.addClean = exports.addCallback = void 0;
var _react = require("react");
const useStore = () => {
  return (0, _react.useRef)({
    callbacksMap: {
      current: undefined
    },
    cleansMap: {
      current: undefined
    }
  }).current;
};
exports.useStore = useStore;
const addCallback = (store, itemKey, callback) => {
  if (!store.callbacksMap.current) {
    store.callbacksMap.current = new Map();
  }
  let callbacks = store.callbacksMap.current.get(itemKey);
  if (!callbacks) {
    callbacks = new Set();
    store.callbacksMap.current.set(itemKey, callbacks);
  }
  callbacks.add(callback);
};
exports.addCallback = addCallback;
const removeCallback = (store, itemKey, callback) => {
  if (!store.callbacksMap.current) return;
  const callbacks = store.callbacksMap.current.get(itemKey);
  callbacks?.delete(callback);
  if (callbacks?.size === 0) {
    store.callbacksMap.current.delete(itemKey);
    if (store.callbacksMap.current.size === 0) {
      store.callbacksMap.current = undefined;
    }
  }
};
exports.removeCallback = removeCallback;
const removeCallbacks = (store, itemKey) => {
  if (!store.callbacksMap.current) return;
  store.callbacksMap.current.delete(itemKey);
  if (store.callbacksMap.current.size === 0) {
    store.callbacksMap.current = undefined;
  }
};
const addClean = (store, itemKey, callback, clean) => {
  if (!store.cleansMap.current) {
    store.cleansMap.current = new Map();
  }
  let cleansWithCallback = store.cleansMap.current.get(itemKey);
  if (!cleansWithCallback) {
    cleansWithCallback = new Map();
    store.cleansMap.current.set(itemKey, cleansWithCallback);
  }
  cleansWithCallback.set(callback, clean);
};
exports.addClean = addClean;
const removeClean = (store, itemKey, callback) => {
  if (!store.cleansMap.current) return;
  const cleansWithCallback = store.cleansMap.current.get(itemKey);
  cleansWithCallback?.delete(callback);
  if (cleansWithCallback?.size === 0) {
    store.cleansMap.current.delete(itemKey);
    if (store.cleansMap.current.size === 0) {
      store.cleansMap.current = undefined;
    }
  }
};
exports.removeClean = removeClean;
const removeCleans = (store, itemKey) => {
  if (!store.cleansMap.current) return;
  store.cleansMap.current.delete(itemKey);
  if (store.cleansMap.current.size === 0) {
    store.cleansMap.current = undefined;
  }
};
const consumeCallbacks = (itemKey, store) => {
  if (!store.callbacksMap.current) return;
  const callbacks = store.callbacksMap.current?.get(itemKey);
  if (!callbacks) return;
  callbacks.forEach(callback => {
    const clean = callback();
    addClean(store, itemKey, callback, clean);
  });
  removeCallbacks(store, itemKey);
};
exports.consumeCallbacks = consumeCallbacks;
const consumeCleans = (itemKey, store) => {
  if (!store.cleansMap.current) return;
  const cleansWithCallback = store.cleansMap.current.get(itemKey);
  if (!cleansWithCallback) return;
  cleansWithCallback.forEach((clean, callback) => {
    if (typeof clean === 'function') {
      clean();
    }

    // give back again
    addCallback(store, itemKey, callback);
  });
  removeCleans(store, itemKey);
};
exports.consumeCleans = consumeCleans;
//# sourceMappingURL=store.js.map